from .permissions import (
    create_permissions,
    check_permission,
    check_permission_bulk,
    get_scopes_ids_by_user_and_scope_name,
    create_visibility,
    initialize,
    ENTITY_TYPES,
    DEFAULT_PERMISSIONS,
    DEFAULT_PERMISSION_BASE_STRING
)

__version__ = "0.3.0"

__all__ = [
    'create_permissions',
    'check_permission',
    'check_permission_bulk',
    'get_scopes_ids_by_user_and_scope_name',
    'create_visibility',
    'initialize',
    'ENTITY_TYPES',
    'DEFAULT_PERMISSIONS',
    'DEFAULT_PERMISSION_BASE_STRING'
] 


from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import Optional, List, Dict, Any


class PlanDuration(str, Enum):
    MONTHLY = "monthly"
    YEARLY = "yearly"


@dataclass
class AddonQuotaInput:
    quota_id: str
    quantity: int


@dataclass
class CreateAddonInput:
    name: str
    price: float
    duration: PlanDuration
    quotas: List[AddonQuotaInput]
    currency: Optional[str] = None  # Default USD


@dataclass
class UpdateAddonInput:
    id: str
    quotas: List[AddonQuotaInput]
    name: Optional[str] = None
    price: Optional[float] = None
    currency: Optional[str] = None
    duration: Optional[PlanDuration] = None
    is_active: Optional[bool] = None


@dataclass
class SubscriptionFeature:
    id: str
    quota: Optional[dict] = None


@dataclass
class Subscription:
    id: str
    status: str
    created_at: datetime
    updated_at: datetime


@dataclass
class AddonSubscription:
    id: str
    current_subscription: Optional[Subscription]
    current_subscription_id: str
    addon_id: str
    quantity: int
    start_date: datetime
    end_date: datetime
    created_at: datetime
    updated_at: datetime


@dataclass
class AddonData:
    id: str
    name: str
    price: float
    currency: str
    duration: PlanDuration
    is_active: bool
    features: List[SubscriptionFeature]
    addon_subscriptions: List[AddonSubscription]
    created_at: datetime
    updated_at: datetime


class Addon:
    """
    Addon management for the Burdenoff Server.
    Handles add-on creation, updates, and retrieval.
    """
    
    def __init__(self, client):
        """
        Initialize Addon module.
        
        Args:
            client: WorkspaceClient instance
        """
        self.client = client
    
    def get_all_addons(self) -> List[AddonData]:
        """
        Get all add-ons.
        
        Returns:
            List of all add-ons
        """
        query = """
        query GetAllAddOns {
            getAllAddOns {
                id
                name
                price
                currency
                duration
                isActive
                features {
                    id
                    quota {
                        id
                    }
                }
                addonSubscriptions {
                    id
                    currentSubscription {
                        id
                        status
                        createdAt
                        updatedAt
                    }
                    currentSubscriptionId
                    addonId
                    quantity
                    startDate
                    endDate
                    createdAt
                    updatedAt
                }
                createdAt
                updatedAt
            }
        }
        """
        
        result = self.client.execute(query)
        
        return [
            self._parse_addon(addon)
            for addon in result['getAllAddOns']
        ]
    
    def get_selected_addons(self, addon_ids: List[str]) -> List[AddonData]:
        """
        Get details for selected add-ons.
        
        Args:
            addon_ids: List of add-on IDs to retrieve
            
        Returns:
            List of add-on details
        """
        query = """
        query GetSelectedAddons($addonIDs: [String!]!) {
            getSelectedAddons(addonIDs: $addonIDs) {
                id
                name
                price
                currency
                duration
                isActive
                features {
                    id
                    quota {
                        id
                    }
                }
                addonSubscriptions {
                    id
                    currentSubscription {
                        id
                        status
                        createdAt
                        updatedAt
                    }
                    currentSubscriptionId
                    addonId
                    quantity
                    startDate
                    endDate
                    createdAt
                    updatedAt
                }
                createdAt
                updatedAt
            }
        }
        """
        
        variables = {
            "addonIDs": addon_ids
        }
        
        result = self.client.execute(query, variables)
        
        return [
            self._parse_addon(addon)
            for addon in result['getSelectedAddons']
        ]
    
    def create_addon(self, input: CreateAddonInput) -> bool:
        """
        Create a new add-on.
        
        Args:
            input: CreateAddonInput with add-on details
            
        Returns:
            Boolean indicating success
        """
        mutation = """
        mutation CreateAddOns($input: createAddOnsInput!) {
            createAddOns(input: $input)
        }
        """
        
        variables = {
            "input": {
                "name": input.name,
                "price": input.price,
                "currency": input.currency,
                "duration": input.duration.value,
                "quotas": [
                    {
                        "quotaId": quota.quota_id,
                        "quantity": quota.quantity
                    }
                    for quota in input.quotas
                ]
            }
        }
        
        result = self.client.execute(mutation, variables)
        return result['createAddOns']
    
    def update_addon(self, input: UpdateAddonInput) -> bool:
        """
        Update an existing add-on.
        
        Args:
            input: UpdateAddonInput with add-on details
            
        Returns:
            Boolean indicating success
        """
        mutation = """
        mutation UpdateAddOnDetails($input: updateAddOnsInput!) {
            updateAddOnDetails(input: $input)
        }
        """
        
        variables = {
            "input": {
                "id": input.id,
                "name": input.name,
                "price": input.price,
                "currency": input.currency,
                "duration": input.duration.value if input.duration else None,
                "isActive": input.is_active,
                "quotas": [
                    {
                        "quotaId": quota.quota_id,
                        "quantity": quota.quantity
                    }
                    for quota in input.quotas
                ]
            }
        }
        
        result = self.client.execute(mutation, variables)
        return result['updateAddOnDetails']
    
    def toggle_addon(self, addon_id: str, status: bool) -> bool:
        """
        Toggle the active status of an add-on.
        
        Args:
            addon_id: ID of the add-on to toggle
            status: New status (True for active, False for inactive)
            
        Returns:
            Boolean indicating success
        """
        mutation = """
        mutation ToggleAddOn($id: ID!, $status: Boolean!) {
            toggleAddOn(id: $id, status: $status)
        }
        """
        
        variables = {
            "id": addon_id,
            "status": status
        }
        
        result = self.client.execute(mutation, variables)
        return result['toggleAddOn']
    
    def _parse_addon(self, addon_data: dict) -> AddonData:
        """
        Parse add-on data from GraphQL response.
        
        Args:
            addon_data: Dict containing add-on data
            
        Returns:
            AddonData object
        """
        return AddonData(
            id=addon_data['id'],
            name=addon_data['name'],
            price=addon_data['price'],
            currency=addon_data['currency'],
            duration=PlanDuration(addon_data['duration']),
            is_active=addon_data['isActive'],
            features=[
                SubscriptionFeature(
                    id=feature['id'],
                    quota=feature.get('quota')
                )
                for feature in addon_data['features']
            ],
            addon_subscriptions=[
                AddonSubscription(
                    id=sub['id'],
                    current_subscription=Subscription(
                        id=sub['currentSubscription']['id'],
                        status=sub['currentSubscription']['status'],
                        created_at=self._parse_datetime(sub['currentSubscription']['createdAt']),
                        updated_at=self._parse_datetime(sub['currentSubscription']['updatedAt'])
                    ) if sub.get('currentSubscription') else None,
                    current_subscription_id=sub['currentSubscriptionId'],
                    addon_id=sub['addonId'],
                    quantity=sub['quantity'],
                    start_date=self._parse_datetime(sub['startDate']),
                    end_date=self._parse_datetime(sub['endDate']),
                    created_at=self._parse_datetime(sub['createdAt']),
                    updated_at=self._parse_datetime(sub['updatedAt'])
                )
                for sub in addon_data['addonSubscriptions']
            ],
            created_at=self._parse_datetime(addon_data['createdAt']),
            updated_at=self._parse_datetime(addon_data['updatedAt'])
        )
    
    def _parse_datetime(self, dt_str: str) -> datetime:
        """
        Parse datetime string from API response.
        
        Args:
            dt_str: Datetime string
            
        Returns:
            datetime object
        """
        return datetime.fromisoformat(dt_str.replace('Z', '+00:00')) 


from burdenoff_common.client import WorkspaceClient
from burdenoff_common.permissions import (
    create_permissions,
    check_permission,
    check_permission_bulk,
    get_scopes_ids_by_user_and_scope_name,
    create_visibility,
    initialize,
    ENTITY_TYPES,
    DEFAULT_PERMISSIONS,
    DEFAULT_PERMISSION_BASE_STRING
)

__version__ = "0.3.0"

__all__ = [
    "WorkspaceClient",
    "create_permissions",
    "check_permission",
    "check_permission_bulk",
    "get_scopes_ids_by_user_and_scope_name",
    "create_visibility",
    "initialize",
    "ENTITY_TYPES",
    "DEFAULT_PERMISSIONS",
    "DEFAULT_PERMISSION_BASE_STRING"
] 


from typing import Dict, Optional, Any, List, Union


class Project:
    """
    Project management operations for Burdenoff Server.
    Handles creating, updating, and managing projects.
    """
    
    def __init__(self, client):
        """
        Initialize Project module.
        
        Args:
            client: WorkspaceClient instance
        """
        self.client = client
    
    def create(self, name: str) -> Dict[str, Any]:
        """
        Create a new project in the current workspace.
        
        Args:
            name: Project name
            
        Returns:
            Dict containing created project details
        """
        query = """
        mutation CreateProject($input: createProjectInput!) {
            createProject(input: $input) {
                id
                name
                workspaceID
                userID
                key
                createdAt
                archived
            }
        }
        """
        
        variables = {
            "input": {
                "name": name
            }
        }
        
        result = self.client.execute(query, variables)
        return result["createProject"]
    
    def get(self, project_id: str) -> Dict[str, Any]:
        """
        Get project details by ID.
        
        Args:
            project_id: ID of the project to retrieve
            
        Returns:
            Dict containing project details
        """
        query = """
        query GetProject($id: ID!) {
            getProject(id: $id) {
                id
                name
                workspaceID
                userID
                key
                createdAt
                archived
                projectMembers {
                    id
                    userID
                    role
                    user {
                        id
                        name
                        email
                    }
                }
            }
        }
        """
        
        variables = {
            "id": project_id
        }
        
        result = self.client.execute(query, variables)
        return result["getProject"]
    
    def list_by_workspaces(self, workspace_ids: List[str]) -> List[Dict[str, Any]]:
        """
        List projects for specified workspaces.
        
        Args:
            workspace_ids: List of workspace IDs
            
        Returns:
            List of workspace projects with their IDs and project lists
        """
        query = """
        query GetWorkspaceProjects($workspaceIDs: [ID!]) {
            getWorkspaceProjects(workspaceIDs: $workspaceIDs) {
                workspaceID
                projects {
                    id
                    name
                    key
                    createdAt
                    archived
                    workspaceID
                    userID
                }
            }
        }
        """
        
        variables = {
            "workspaceIDs": workspace_ids
        }
        
        result = self.client.execute(query, variables)
        return result["getWorkspaceProjects"]
    
    def update(self, project_id: str, name: str) -> Dict[str, Any]:
        """
        Update project details.
        
        Args:
            project_id: ID of the project to update
            name: New project name
            
        Returns:
            Dict containing updated project details
        """
        query = """
        mutation EditProject($input: editProjectInput!) {
            editProject(input: $input) {
                id
                name
                workspaceID
                key
                createdAt
                archived
            }
        }
        """
        
        variables = {
            "input": {
                "id": project_id,
                "name": name
            }
        }
        
        result = self.client.execute(query, variables)
        return result["editProject"]
    
    def archive(self, project_id: str) -> Dict[str, Any]:
        """
        Archive a project.
        
        Args:
            project_id: ID of the project to archive
            
        Returns:
            Dict containing archived project details
        """
        query = """
        mutation ArchiveProject($id: ID!) {
            archiveProject(id: $id) {
                id
                name
                workspaceID
                archived
            }
        }
        """
        
        variables = {
            "id": project_id
        }
        
        result = self.client.execute(query, variables)
        return result["archiveProject"]
    
    def unarchive(self, project_id: str) -> Dict[str, Any]:
        """
        Unarchive a project.
        
        Args:
            project_id: ID of the project to unarchive
            
        Returns:
            Dict containing unarchived project details
        """
        query = """
        mutation UnarchiveProject($id: ID!) {
            unarchiveProject(id: $id) {
                id
                name
                workspaceID
                archived
            }
        }
        """
        
        variables = {
            "id": project_id
        }
        
        result = self.client.execute(query, variables)
        return result["unarchiveProject"]
    
    def switch_project(self, project_id: str) -> str:
        """
        Switch the active project.
        
        Args:
            project_id: ID of the project to switch to
            
        Returns:
            String confirmation message
        """
        query = """
        mutation SwitchProject($projID: ID!) {
            switchProject(projID: $projID)
        }
        """
        
        variables = {
            "projID": project_id
        }
        
        result = self.client.execute(query, variables)
        return result["switchProject"]
    
    def update_member_role(self, project_id: str, user_id: str, role: str) -> Dict[str, Any]:
        """
        Update the role of a project member.
        
        Args:
            project_id: Project ID
            user_id: User ID
            role: New role to assign
            
        Returns:
            Dict containing updated project member details
        """
        query = """
        mutation UpdateProjectMemberRole($input: UpdateProjectMemberRoleInput!) {
            updateProjectMemberRole(input: $input) {
                id
                projectID
                userID
                role
                user {
                    id
                    name
                    email
                }
            }
        }
        """
        
        variables = {
            "input": {
                "projectID": project_id,
                "userID": user_id,
                "role": role
            }
        }
        
        result = self.client.execute(query, variables)
        return result["updateProjectMemberRole"]
    
    def remove_member(self, project_id: str, user_id: str, member_id: str) -> bool:
        """
        Remove a member from a project.
        
        Args:
            project_id: Project ID
            user_id: User ID to remove
            member_id: Membership ID
            
        Returns:
            Boolean indicating if removal was successful
        """
        query = """
        mutation RemoveProjectMember($input: RemoveProjectMemberInput!) {
            removeProjectMember(input: $input)
        }
        """
        
        variables = {
            "input": {
                "projectID": project_id,
                "userID": user_id,
                "id": member_id
            }
        }
        
        result = self.client.execute(query, variables)
        return result["removeProjectMember"] 


from typing import List, Dict, Optional, TypedDict, Any, Union
import os
from gql import gql, Client
from gql.transport.requests import RequestsHTTPTransport

# Type definitions
class PermissionResponse(TypedDict):
    hasPermission: bool

class BulkPermissionResponse(TypedDict):
    allow: bool
    permission: str
    scope: str

class PermissionInput(TypedDict):
    permission: str
    scope: str

class PermissionResult(TypedDict):
    allow: bool
    permission: str
    scope: str

# Constants
ENTITY_TYPES = {
    'team': 'team',
    'billingAccount': 'billingAccount',
    'project': 'project',
    'workspace': 'workspace',
    'knowledgeBase': 'knowledgeBase',
    'newsLetter': 'newsLetter',
    'plan': 'plan',
    'supportTicket': 'supportTicket',
    'workflow': 'workflow',
    'bot': 'bot'
}

DEFAULT_PERMISSIONS = {
    'create': 'create',
    'read': 'read',
    'update': 'update',
    'delete': 'delete',
    'all': 'all'
}

DEFAULT_PERMISSION_BASE_STRING = 'https://permissions.burdenoff.com'

class PermissionClient:
    def __init__(self, graphql_endpoint: str, permission_base_string: Optional[str] = None):
        """
        Initialize the Permission client with a GraphQL endpoint and optional permission base string.
        
        Args:
            graphql_endpoint (str): The GraphQL endpoint URL
            permission_base_string (str, optional): The base string for permissions. 
                                                  Defaults to 'https://permissions.burdenoff.com'
        """
        self.graphql_endpoint = graphql_endpoint
        self.permission_base_string = permission_base_string or DEFAULT_PERMISSION_BASE_STRING

    def get_client(self, token: str) -> Client:
        """Create and return a GraphQL client with the given token."""
        transport = RequestsHTTPTransport(
            url=self.graphql_endpoint,
            headers={'Authorization': f'{token}'}
        )
        return Client(transport=transport, fetch_schema_from_transport=True)

    async def create_permissions(
        self,
        entity_id: str,
        entity_type: str,
        scope_name: str,
        token: str,
        child_entity_types: Optional[List[str]] = None,
        custom_permissions: Optional[List[str]] = None,
        workspace_id: Optional[str] = None
    ) -> Optional[Dict]:
        """
        Create permissions for an entity.
        
        Args:
            entity_id (str): The ID of the entity
            entity_type (str): The type of entity (should be one of ENTITY_TYPES)
            scope_name (str): The name of the scope
            token (str): Authentication token
            child_entity_types (List[str], optional): List of child entity types
            custom_permissions (List[str], optional): List of custom permissions
            workspace_id (str, optional): The workspace ID if applicable
            
        Returns:
            Optional[Dict]: The created permission or None if failed
        """
        try:
            client = self.get_client(token)
            mutation = gql("""
                mutation createPermission(
                    $childEntityTypes: [String!]
                    $customPermissions: [String!]
                    $entityID: ID!
                    $entityType: String!
                    $scopeName: String!
                ) {
                    createPermission(
                        entityID: $entityID
                        scopeName: $scopeName
                        childEntityTypes: $childEntityTypes
                        customPermissions: $customPermissions
                        entityType: $entityType
                    )
                }
            """)
            
            variables = {
                'entityID': entity_id,
                'scopeName': scope_name,
                'childEntityTypes': child_entity_types,
                'customPermissions': custom_permissions,
                'entityType': entity_type
            }
            
            result = client.execute(mutation, variable_values=variables)
            return result
        except Exception as e:
            print(f"Create permission failed: {str(e)}")
            return None

    async def check_permission(
        self,
        permission: str,
        scope: str,
        token: str
    ) -> PermissionResponse:
        """
        Check if a permission is allowed for a specific scope.
        
        Args:
            permission (str): The permission to check
            scope (str): The scope to check the permission in
            token (str): Authentication token
            
        Returns:
            PermissionResponse: Dictionary with hasPermission field indicating if permission is allowed
        """
        try:
            client = self.get_client(token)
            query = gql("""
                query hasPermission($permission: String!, $scope: String!) {
                    hasPermission(permission: $permission, scope: $scope)
                }
            """)
            
            variables = {
                'permission': permission,
                'scope': scope
            }
            
            result = client.execute(query, variable_values=variables)
            return result
        except Exception as e:
            print(f"Permission check error: {str(e)}")
            return {'hasPermission': False}

    async def check_permission_bulk(
        self,
        permission_inputs: List[PermissionInput],
        token: str
    ) -> List[PermissionResult]:
        """
        Verify multiple permissions in bulk.
        
        Args:
            permission_inputs (List[PermissionInput]): List of permission inputs, each containing
                                                     permission and scope to verify
            token (str): Authentication token
            
        Returns:
            List[PermissionResult]: List of dictionaries containing allow status, permission,
                                  and scope for each input
        """
        try:
            client = self.get_client(token)
            query = gql("""
                query verifyPermissions($input: [PermissionInput!]!) {
                    verifyPermission(input: $input) {
                        allow
                        permission
                        scope
                    }
                }
            """)
            
            variables = {
                'input': permission_inputs
            }
            
            result = client.execute(query, variable_values=variables)
            return result.get('verifyPermission', [
                {
                    'allow': False,
                    'permission': input_item['permission'],
                    'scope': input_item['scope']
                }
                for input_item in permission_inputs
            ])
        except Exception as e:
            print(f"Permission verification failed: {str(e)}")
            return [
                {
                    'allow': False,
                    'permission': input_item['permission'],
                    'scope': input_item['scope']
                }
                for input_item in permission_inputs
            ]

    async def get_scopes_ids_by_user_and_scope_name(
        self,
        permission: str,
        scope_type: str,
        token: str
    ) -> List[str]:
        """
        Get all scope IDs for a user and scope type.
        
        Args:
            permission (str): The permission to check
            scope_type (str): The type of scope
            token (str): Authentication token
            
        Returns:
            List[str]: List of scope IDs
        """
        try:
            client = self.get_client(token)
            query = gql("""
                query getAllScopesByUserAndScopeType(
                    $permission: String!
                    $permissionBaseString: String!
                    $scopeType: String!
                ) {
                    getAllScopesByUserAndScopeType(
                        permission: $permission
                        permissionBaseString: $permissionBaseString
                        scopeType: $scopeType
                    )
                }
            """)
            
            variables = {
                'permission': permission,
                'scopeType': scope_type,
                'permissionBaseString': self.permission_base_string
            }
            
            result = client.execute(query, variable_values=variables)
            return result.get('getAllScopesByUserAndScopeType', [])
        except Exception as e:
            print(f"Error getting scopes: {str(e)}")
            return []

    async def create_visibility(
        self,
        permission: str,
        scope: str,
        visibility: str,
        visibility_id: str,
        token: str
    ) -> Optional[Dict]:
        """
        Create a visibility entry.
        
        Args:
            permission (str): The permission to set visibility for
            scope (str): The scope of the visibility
            visibility (str): The visibility value
            visibility_id (str): The ID for the visibility
            token (str): Authentication token
            
        Returns:
            Optional[Dict]: The created visibility entry or None if failed
        """
        try:
            client = self.get_client(token)
            mutation = gql("""
                mutation CreateVisibility(
                    $permission: String!,
                    $scope: String!,
                    $visibility: String!,
                    $visibility_id: String!
                ) {
                    createVisibility(
                        input: {
                            visibility: $visibility,
                            visibility_id: $visibility_id,
                            permission: $permission,
                            scope: $scope
                        }
                    ) {
                        id
                    }
                }
            """)
            
            variables = {
                'permission': permission,
                'scope': scope,
                'visibility': visibility,
                'visibility_id': visibility_id
            }
            
            result = client.execute(mutation, variable_values=variables)
            return result.get('createVisibility')
        except Exception as e:
            print(f"Error creating visibility: {str(e)}")
            return None

# Create convenience functions that use a default client
_default_client = None

def initialize(graphql_endpoint: str, permission_base_string: Optional[str] = None):
    """
    Initialize the default client with the GraphQL endpoint and optional permission base string.
    
    Args:
        graphql_endpoint (str): The GraphQL endpoint URL
        permission_base_string (str, optional): The base string for permissions. 
                                              Defaults to 'https://permissions.burdenoff.com'
    """
    global _default_client
    _default_client = PermissionClient(graphql_endpoint, permission_base_string)

def get_client() -> PermissionClient:
    """Get the default permission client."""
    if _default_client is None:
        raise RuntimeError("Permission client not initialized. Call initialize() first.")
    return _default_client

# Convenience functions that use the default client
async def create_permissions(*args, **kwargs):
    return await get_client().create_permissions(*args, **kwargs)

async def check_permission(*args, **kwargs):
    return await get_client().check_permission(*args, **kwargs)

async def check_permission_bulk(*args, **kwargs):
    return await get_client().check_permission_bulk(*args, **kwargs)

async def get_scopes_ids_by_user_and_scope_name(*args, **kwargs):
    return await get_client().get_scopes_ids_by_user_and_scope_name(*args, **kwargs)

async def create_visibility(*args, **kwargs):
    return await get_client().create_visibility(*args, **kwargs) 


from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import Optional, List, Dict, Any


class PlanDuration(str, Enum):
    MONTHLY = "monthly"
    YEARLY = "yearly"


class SubscriptionStatus(str, Enum):
    ACTIVE = "active"
    CANCELED = "canceled"
    EXPIRED = "expired"


@dataclass
class SubscriptionFeature:
    id: str


@dataclass
class SubscriptionEntitlement:
    id: str


@dataclass
class QuotaAssignment:
    id: str


@dataclass
class Subscription:
    id: str
    billing_account_id: str
    plan_id: str
    status: SubscriptionStatus
    start_date: datetime
    end_date: datetime
    credits_awarded: float
    recurrence: bool
    duration: PlanDuration
    quotas: List[QuotaAssignment]
    created_at: datetime
    updated_at: datetime


@dataclass
class PlanData:
    id: str
    name: str
    price: float
    currency: str
    duration: PlanDuration
    is_active: bool
    features: List[SubscriptionFeature]
    subscriptions: List[Subscription]
    item_entitlements: List[SubscriptionEntitlement]
    created_at: datetime
    updated_at: datetime


@dataclass
class PlanQuotaInput:
    quota_id: str
    quantity: int


@dataclass
class CreatePlanInput:
    name: str
    price: float
    duration: PlanDuration
    quotas: List[PlanQuotaInput]
    currency: Optional[str] = None  # Default USD


@dataclass
class UpdatePlanInput:
    id: str
    quotas: List[PlanQuotaInput]
    name: Optional[str] = None
    price: Optional[float] = None
    currency: Optional[str] = None
    duration: Optional[PlanDuration] = None
    is_active: Optional[bool] = None


class Plan:
    """
    Plan management for the Burdenoff Server.
    Handles plan creation, updates, and retrieval.
    """
    
    def __init__(self, client):
        """
        Initialize Plan module.
        
        Args:
            client: WorkspaceClient instance
        """
        self.client = client
    
    def get_all_plans(self) -> List[PlanData]:
        """
        Get all plans.
        
        Returns:
            List of all plans
        """
        query = """
        query GetAllPlans {
            getAllPlans {
                id
                name
                price
                currency
                duration
                isActive
                features {
                    id
                }
                subscriptions {
                    id
                    billingAccountId
                    planId
                    status
                    startDate
                    endDate
                    creditsAwarded
                    recurrence
                    duration
                    quotas {
                        id
                    }
                    createdAt
                    updatedAt
                }
                itemEntitlements {
                    id
                }
                createdAt
                updatedAt
            }
        }
        """
        
        result = self.client.execute(query)
        
        return [
            self._parse_plan(plan)
            for plan in result['getAllPlans']
        ]
    
    def get_plan(self, plan_id: str) -> PlanData:
        """
        Get details for a specific plan.
        
        Args:
            plan_id: ID of the plan to retrieve
            
        Returns:
            Plan details
        """
        query = """
        query GetPlan($id: ID!) {
            getPlan(id: $id) {
                id
                name
                price
                currency
                duration
                isActive
                features {
                    id
                }
                subscriptions {
                    id
                    billingAccountId
                    planId
                    status
                    startDate
                    endDate
                    creditsAwarded
                    recurrence
                    duration
                    quotas {
                        id
                    }
                    createdAt
                    updatedAt
                }
                itemEntitlements {
                    id
                }
                createdAt
                updatedAt
            }
        }
        """
        
        variables = {
            "id": plan_id
        }
        
        result = self.client.execute(query, variables)
        
        return self._parse_plan(result['getPlan'])
    
    def create_plan(self, input: CreatePlanInput) -> bool:
        """
        Create a new plan.
        
        Args:
            input: CreatePlanInput with plan details
            
        Returns:
            Boolean indicating success
        """
        mutation = """
        mutation CreatePlan($input: createPlanInput!) {
            createPlan(input: $input)
        }
        """
        
        variables = {
            "input": {
                "name": input.name,
                "price": input.price,
                "currency": input.currency,
                "duration": input.duration.value,
                "quotas": [
                    {
                        "quotaId": quota.quota_id,
                        "quantity": quota.quantity
                    }
                    for quota in input.quotas
                ]
            }
        }
        
        result = self.client.execute(mutation, variables)
        return result['createPlan']
    
    def update_plan(self, input: UpdatePlanInput) -> bool:
        """
        Update an existing plan.
        
        Args:
            input: UpdatePlanInput with plan details
            
        Returns:
            Boolean indicating success
        """
        mutation = """
        mutation UpdatePlan($input: updatePlanInput!) {
            updatePlan(input: $input)
        }
        """
        
        variables = {
            "input": {
                "id": input.id,
                "name": input.name,
                "price": input.price,
                "currency": input.currency,
                "duration": input.duration.value if input.duration else None,
                "isActive": input.is_active,
                "quotas": [
                    {
                        "quotaId": quota.quota_id,
                        "quantity": quota.quantity
                    }
                    for quota in input.quotas
                ]
            }
        }
        
        result = self.client.execute(mutation, variables)
        return result['updatePlan']
    
    def toggle_plan(self, plan_id: str, status: bool) -> bool:
        """
        Toggle the active status of a plan.
        
        Args:
            plan_id: ID of the plan to toggle
            status: New status (True for active, False for inactive)
            
        Returns:
            Boolean indicating success
        """
        mutation = """
        mutation TogglePlan($planID: ID!, $status: Boolean!) {
            togglePlan(planID: $planID, status: $status)
        }
        """
        
        variables = {
            "planID": plan_id,
            "status": status
        }
        
        result = self.client.execute(mutation, variables)
        return result['togglePlan']
    
    def _parse_plan(self, plan_data: dict) -> PlanData:
        """
        Parse plan data from GraphQL response.
        
        Args:
            plan_data: Dict containing plan data
            
        Returns:
            PlanData object
        """
        return PlanData(
            id=plan_data['id'],
            name=plan_data['name'],
            price=plan_data['price'],
            currency=plan_data['currency'],
            duration=PlanDuration(plan_data['duration']),
            is_active=plan_data['isActive'],
            features=[SubscriptionFeature(id=f['id']) for f in plan_data['features']],
            subscriptions=[
                Subscription(
                    id=sub['id'],
                    billing_account_id=sub['billingAccountId'],
                    plan_id=sub['planId'],
                    status=SubscriptionStatus(sub['status']),
                    start_date=self._parse_datetime(sub['startDate']),
                    end_date=self._parse_datetime(sub['endDate']),
                    credits_awarded=sub['creditsAwarded'],
                    recurrence=sub['recurrence'],
                    duration=PlanDuration(sub['duration']),
                    quotas=[QuotaAssignment(id=q['id']) for q in sub['quotas']],
                    created_at=self._parse_datetime(sub['createdAt']),
                    updated_at=self._parse_datetime(sub['updatedAt'])
                )
                for sub in plan_data['subscriptions']
            ],
            item_entitlements=[SubscriptionEntitlement(id=e['id']) for e in plan_data['itemEntitlements']],
            created_at=self._parse_datetime(plan_data['createdAt']),
            updated_at=self._parse_datetime(plan_data['updatedAt'])
        )
    
    def _parse_datetime(self, dt_str: str) -> datetime:
        """
        Parse datetime string from API response.
        
        Args:
            dt_str: Datetime string
            
        Returns:
            datetime object
        """
        return datetime.fromisoformat(dt_str.replace('Z', '+00:00')) 


from typing import List, Dict, Optional, Any
from datetime import datetime
from enum import Enum
from dataclasses import dataclass


class PaymentMethodType(str, Enum):
    CREDIT_CARD = "credit_card"
    DEBIT_CARD = "debit_card"
    PAYMENT_GATEWAY = "payment_gateway"
    NET_BANKING = "net_banking"
    UPI = "upi"
    WALLET = "wallet"
    OTHER = "other"


class TransactionStatus(str, Enum):
    PENDING = "pending"
    SUCCESS = "success"
    FAILED = "failed"


class TransactionType(str, Enum):
    PAYMENT = "payment"
    REFUND = "refund"
    ADJUSTMENT = "adjustment"


class InvoiceStatus(str, Enum):
    DRAFT = "draft"
    PAID = "paid"


@dataclass
class Transaction:
    id: str
    billing_account_id: str
    payment_method_id: Optional[str]
    amount: float
    currency: str
    description: Optional[str]
    status: TransactionStatus
    type: TransactionType
    discount_id: Optional[str]
    created_at: datetime
    updated_at: datetime


@dataclass
class Invoice:
    id: str
    billing_account_id: str
    total_amount: float
    currency: str
    period_start: datetime
    period_end: datetime
    status: InvoiceStatus
    tax_amount: float
    tax_details: Optional[Dict[str, Any]]
    created_at: datetime
    updated_at: datetime


@dataclass
class PollingStatus:
    status: str


@dataclass
class PDF:
    pdf: str


@dataclass
class AddOnInputPayment:
    id: str
    quantity: Optional[int]


@dataclass
class CreateRazorpayOrderInput:
    billing_account_id: str
    plan_id: Optional[str]
    store_item_id: Optional[str]
    add_on_ids: Optional[List[AddOnInputPayment]]
    recurrence: Optional[bool]


@dataclass
class CreateStripeOrderInput:
    billing_account_id: str
    plan_id: Optional[str]
    store_item_id: Optional[str]
    add_on_ids: Optional[List[AddOnInputPayment]]
    recurrence: Optional[bool]


@dataclass
class CreateRazorpayOrderResponse:
    amount: str
    currency: str
    order_id: str
    key: str
    transaction_id: str


@dataclass
class CreateStripeOrderResponse:
    client_secret: str
    transaction_id: str
    key: str
    amount: str


class Payment:
    """
    Payment operations for the Burdenoff Server.
    Handles transactions, invoices, and payment processing.
    """
    
    def __init__(self, client):
        """
        Initialize Payment module.
        
        Args:
            client: WorkspaceClient instance
        """
        self.client = client
    
    def get_billing_account_transactions(self, billing_account_id: str, days: int) -> List[Transaction]:
        """
        Get transactions for a billing account within a specific time period.
        
        Args:
            billing_account_id: ID of the billing account
            days: Number of days to look back
            
        Returns:
            List of transactions
        """
        query = """
        query GetBillingAccountTransactions($billingAccountID: ID!, $days: Int!) {
            getBillingAccountTransactions(billingAccountID: $billingAccountID, days: $days) {
                id
                billingAccountId
                paymentMethodId
                amount
                currency
                description
                status
                type
                discountId
                createdAt
                updatedAt
            }
        }
        """
        
        variables = {
            "billingAccountID": billing_account_id,
            "days": days
        }
        
        result = self.client.execute(query, variables)
        
        return [
            Transaction(
                id=tx['id'],
                billing_account_id=tx['billingAccountId'],
                payment_method_id=tx.get('paymentMethodId'),
                amount=tx['amount'],
                currency=tx['currency'],
                description=tx.get('description'),
                status=TransactionStatus(tx['status']),
                type=TransactionType(tx['type']),
                discount_id=tx.get('discountId'),
                created_at=self._parse_datetime(tx['createdAt']),
                updated_at=self._parse_datetime(tx['updatedAt'])
            )
            for tx in result['getBillingAccountTransactions']
        ]
    
    def get_transaction_details(self, transaction_id: str) -> Transaction:
        """
        Get details for a specific transaction.
        
        Args:
            transaction_id: ID of the transaction
            
        Returns:
            Transaction details
        """
        query = """
        query GetTransactionDetails($transactionID: ID!) {
            getTransactionDetails(transactionID: $transactionID) {
                id
                billingAccountId
                paymentMethodId
                amount
                currency
                description
                status
                type
                discountId
                createdAt
                updatedAt
            }
        }
        """
        
        variables = {
            "transactionID": transaction_id
        }
        
        result = self.client.execute(query, variables)
        tx = result['getTransactionDetails']
        
        return Transaction(
            id=tx['id'],
            billing_account_id=tx['billingAccountId'],
            payment_method_id=tx.get('paymentMethodId'),
            amount=tx['amount'],
            currency=tx['currency'],
            description=tx.get('description'),
            status=TransactionStatus(tx['status']),
            type=TransactionType(tx['type']),
            discount_id=tx.get('discountId'),
            created_at=self._parse_datetime(tx['createdAt']),
            updated_at=self._parse_datetime(tx['updatedAt'])
        )
    
    def get_invoice(self, invoice_id: str) -> Invoice:
        """
        Get details for a specific invoice.
        
        Args:
            invoice_id: ID of the invoice
            
        Returns:
            Invoice details
        """
        query = """
        query GetInvoice($invoiceID: ID!) {
            getInvoice(invoiceID: $invoiceID) {
                id
                billingAccountId
                totalAmount
                currency
                periodStart
                periodEnd
                status
                taxAmount
                taxDetails
                createdAt
                updatedAt
            }
        }
        """
        
        variables = {
            "invoiceID": invoice_id
        }
        
        result = self.client.execute(query, variables)
        inv = result['getInvoice']
        
        return Invoice(
            id=inv['id'],
            billing_account_id=inv['billingAccountId'],
            total_amount=inv['totalAmount'],
            currency=inv['currency'],
            period_start=self._parse_datetime(inv['periodStart']),
            period_end=self._parse_datetime(inv['periodEnd']),
            status=InvoiceStatus(inv['status']),
            tax_amount=inv['taxAmount'],
            tax_details=inv.get('taxDetails'),
            created_at=self._parse_datetime(inv['createdAt']),
            updated_at=self._parse_datetime(inv['updatedAt'])
        )
    
    def get_polling_status(self, transaction_id: str) -> PollingStatus:
        """
        Get current status of a transaction.
        
        Args:
            transaction_id: ID of the transaction
            
        Returns:
            Status information
        """
        query = """
        query GetPollingStatus($transactionID: String!) {
            pollingStatus(transactionID: $transactionID) {
                status
            }
        }
        """
        
        variables = {
            "transactionID": transaction_id
        }
        
        result = self.client.execute(query, variables)
        return PollingStatus(status=result['pollingStatus']['status'])
    
    def get_invoice_pdf(self, invoice_id: str) -> PDF:
        """
        Get PDF content for an invoice.
        
        Args:
            invoice_id: ID of the invoice
            
        Returns:
            PDF content
        """
        query = """
        query GetInvoicePdf($invoiceID: ID!) {
            getInvoicePdf(invoiceID: $invoiceID) {
                pdf
            }
        }
        """
        
        variables = {
            "invoiceID": invoice_id
        }
        
        result = self.client.execute(query, variables)
        return PDF(pdf=result['getInvoicePdf']['pdf'])
    
    def generate_invoice_for_billing_account(
        self,
        billing_account_id: str,
        start_date: datetime,
        end_date: datetime
    ) -> Invoice:
        """
        Generate an invoice for a billing account in a specific period.
        
        Args:
            billing_account_id: ID of the billing account
            start_date: Start date for the invoice period
            end_date: End date for the invoice period
            
        Returns:
            Generated invoice
        """
        mutation = """
        mutation GenerateInvoiceForBillingAccount(
            $billingAccountID: ID!,
            $startDate: DateTime!,
            $endDate: DateTime!
        ) {
            generateInvoiceForBillingAccount(
                billingAccountID: $billingAccountID,
                startDate: $startDate,
                endDate: $endDate
            ) {
                id
                billingAccountId
                totalAmount
                currency
                periodStart
                periodEnd
                status
                taxAmount
                taxDetails
                createdAt
                updatedAt
            }
        }
        """
        
        variables = {
            "billingAccountID": billing_account_id,
            "startDate": start_date.isoformat(),
            "endDate": end_date.isoformat()
        }
        
        result = self.client.execute(mutation, variables)
        inv = result['generateInvoiceForBillingAccount']
        
        return Invoice(
            id=inv['id'],
            billing_account_id=inv['billingAccountId'],
            total_amount=inv['totalAmount'],
            currency=inv['currency'],
            period_start=self._parse_datetime(inv['periodStart']),
            period_end=self._parse_datetime(inv['periodEnd']),
            status=InvoiceStatus(inv['status']),
            tax_amount=inv['taxAmount'],
            tax_details=inv.get('taxDetails'),
            created_at=self._parse_datetime(inv['createdAt']),
            updated_at=self._parse_datetime(inv['updatedAt'])
        )
    
    def generate_invoice_for_transaction(self, transaction_id: str) -> Invoice:
        """
        Generate an invoice for a specific transaction.
        
        Args:
            transaction_id: ID of the transaction
            
        Returns:
            Generated invoice
        """
        mutation = """
        mutation GenerateInvoiceForTransaction($transactionID: ID!) {
            generateInvoiceForTransaction(transactionID: $transactionID) {
                id
                billingAccountId
                totalAmount
                currency
                periodStart
                periodEnd
                status
                taxAmount
                taxDetails
                createdAt
                updatedAt
            }
        }
        """
        
        variables = {
            "transactionID": transaction_id
        }
        
        result = self.client.execute(mutation, variables)
        inv = result['generateInvoiceForTransaction']
        
        return Invoice(
            id=inv['id'],
            billing_account_id=inv['billingAccountId'],
            total_amount=inv['totalAmount'],
            currency=inv['currency'],
            period_start=self._parse_datetime(inv['periodStart']),
            period_end=self._parse_datetime(inv['periodEnd']),
            status=InvoiceStatus(inv['status']),
            tax_amount=inv['taxAmount'],
            tax_details=inv.get('taxDetails'),
            created_at=self._parse_datetime(inv['createdAt']),
            updated_at=self._parse_datetime(inv['updatedAt'])
        )
    
    def create_razorpay_order(self, input: CreateRazorpayOrderInput) -> CreateRazorpayOrderResponse:
        """
        Create a new order using Razorpay.
        
        Args:
            input: Order creation parameters
            
        Returns:
            Order details
        """
        mutation = """
        mutation CreateRazorpayOrder($input: createRazorpayOrderInput!) {
            createRazorpayOrder(input: $input) {
                amount
                currency
                orderID
                key
                transactionID
            }
        }
        """
        
        variables = {
            "input": {
                "billingAccountId": input.billing_account_id,
                "planId": input.plan_id,
                "storeItemId": input.store_item_id,
                "addOnInput": [{"id": addon.id, "quantity": addon.quantity} for addon in input.add_on_ids] if input.add_on_ids else None,
                "recurrence": input.recurrence
            }
        }
        
        result = self.client.execute(mutation, variables)
        order = result['createRazorpayOrder']
        
        return CreateRazorpayOrderResponse(
            amount=order['amount'],
            currency=order['currency'],
            order_id=order['orderID'],
            key=order['key'],
            transaction_id=order['transactionID']
        )
    
    def create_stripe_order(self, input: CreateStripeOrderInput) -> CreateStripeOrderResponse:
        """
        Create a new order using Stripe.
        
        Args:
            input: Order creation parameters
            
        Returns:
            Order details
        """
        mutation = """
        mutation CreateStripeOrder($input: createStripeOrderInput!) {
            createStripeOrder(input: $input) {
                clientSecret
                key
                transactionID
            }
        }
        """
        
        variables = {
            "input": {
                "billingAccountId": input.billing_account_id,
                "planId": input.plan_id,
                "storeItemId": input.store_item_id,
                "addOnInput": [{"id": addon.id, "quantity": addon.quantity} for addon in input.add_on_ids] if input.add_on_ids else None,
                "recurrence": input.recurrence
            }
        }
        
        result = self.client.execute(mutation, variables)
        order = result['createStripeOrder']
        
        return CreateStripeOrderResponse(
            client_secret=order['clientSecret'],
            key=order['key'],
            transaction_id=order['transactionID'],
            amount=order.get('amount', "0")  # Default to "0" if amount is not provided
        )
    
    def _parse_datetime(self, dt_str: str) -> datetime:
        """Parse datetime string from the API"""
        try:
            # Try the dateutil parser first
            from dateutil.parser import parse
            return parse(dt_str)
        except (ImportError, ValueError):
            # Fallback to basic datetime parsing
            try:
                return datetime.fromisoformat(dt_str.replace('Z', '+00:00'))
            except ValueError:
                # Last resort fallback
                return datetime.utcnow() 


from typing import List, Optional, Dict, Any
from datetime import datetime
from enum import Enum
from dataclasses import dataclass


class QuotaEntityType(str, Enum):
    WORKSPACE = "workspace"
    PROJECT = "project"


@dataclass
class Plan:
    id: str
    name: str


@dataclass
class Addon:
    id: str
    name: str


@dataclass
class SubscriptionFeature:
    id: str
    plan_id: Optional[str]
    plan: Optional[Plan]
    addon_id: Optional[str]
    addon: Optional[Addon]
    quota_id: Optional[str]
    quota: Optional['Quota']


@dataclass
class Quota:
    id: str
    name: str
    limits: Dict[str, Any]  # JSON object defining quota limits
    is_active: bool
    reusable: bool
    subscriptions: List[SubscriptionFeature]
    created_at: datetime
    updated_at: datetime


@dataclass
class CreateQuotaInput:
    name: str
    limits: Dict[str, Any]  # JSON object defining quota limits
    reusable: bool


@dataclass
class UpdateQuotaInput:
    id: str
    is_active: bool
    name: Optional[str] = None
    limits: Optional[Dict[str, Any]] = None
    reusable: Optional[bool] = None


class Quota:
    """
    Quota management for the Burdenoff Server.
    Handles quota creation, updates, and retrieval.
    """
    
    def __init__(self, client):
        """
        Initialize Quota module.
        
        Args:
            client: WorkspaceClient instance
        """
        self.client = client
    
    def get_all_quotas(self) -> List[Quota]:
        """
        Get all quotas.
        
        Returns:
            List of all quotas
        """
        query = """
        query GetAllQuotas {
            getAllQuotas {
                id
                name
                limits
                isActive
                reusable
                subscriptions {
                    id
                    planId
                    plan {
                        id
                        name
                    }
                    addonId
                    addOn {
                        id
                        name
                    }
                    quotaId
                    quota {
                        id
                        name
                    }
                }
                createdAt
                updatedAt
            }
        }
        """
        
        result = self.client.execute(query)
        
        return [
            self._parse_quota(quota)
            for quota in result['getAllQuotas']
        ]
    
    def get_all_quotas_of_sub(self, subscription_id: str) -> List[Quota]:
        """
        Get all quotas of a subscription.
        
        Args:
            subscription_id: ID of the subscription
            
        Returns:
            List of quotas associated with the subscription
        """
        query = """
        query GetAllQuotasOfSub($subscriptionId: ID!) {
            getAllQuotasOfSub(subscriptionId: $subscriptionId) {
                id
                name
                limits
                isActive
                reusable
                subscriptions {
                    id
                    planId
                    plan {
                        id
                        name
                    }
                    addonId
                    addOn {
                        id
                        name
                    }
                    quotaId
                    quota {
                        id
                        name
                    }
                }
                createdAt
                updatedAt
            }
        }
        """
        
        variables = {
            "subscriptionId": subscription_id
        }
        
        result = self.client.execute(query, variables)
        
        return [
            self._parse_quota(quota)
            for quota in result['getAllQuotasOfSub']
        ]
    
    def create_quota(self, input: CreateQuotaInput) -> bool:
        """
        Create a new quota.
        
        Args:
            input: CreateQuotaInput with quota details
            
        Returns:
            Boolean indicating success
        """
        mutation = """
        mutation CreateQuota($input: createQuotaInput!) {
            createQuota(input: $input)
        }
        """
        
        variables = {
            "input": {
                "name": input.name,
                "limits": input.limits,
                "reusable": input.reusable
            }
        }
        
        result = self.client.execute(mutation, variables)
        return result['createQuota']
    
    def update_quota(self, input: UpdateQuotaInput) -> bool:
        """
        Update an existing quota.
        
        Args:
            input: UpdateQuotaInput with quota details
            
        Returns:
            Boolean indicating success
        """
        mutation = """
        mutation UpdateQuota($input: updateQuotaInput!) {
            updateQuota(input: $input)
        }
        """
        
        variables = {
            "input": {
                "id": input.id,
                "name": input.name,
                "limits": input.limits,
                "reusable": input.reusable,
                "isActive": input.is_active
            }
        }
        
        result = self.client.execute(mutation, variables)
        return result['updateQuota']
    
    def _parse_quota(self, quota_data: dict) -> Quota:
        """
        Parse quota data from GraphQL response.
        
        Args:
            quota_data: Dict containing quota data
            
        Returns:
            Quota object
        """
        return Quota(
            id=quota_data['id'],
            name=quota_data['name'],
            limits=quota_data['limits'],
            is_active=quota_data['isActive'],
            reusable=quota_data['reusable'],
            subscriptions=[
                SubscriptionFeature(
                    id=sub['id'],
                    plan_id=sub.get('planId'),
                    plan=Plan(
                        id=sub['plan']['id'],
                        name=sub['plan']['name']
                    ) if sub.get('plan') else None,
                    addon_id=sub.get('addonId'),
                    addon=Addon(
                        id=sub['addOn']['id'],
                        name=sub['addOn']['name']
                    ) if sub.get('addOn') else None,
                    quota_id=sub.get('quotaId'),
                    quota=Quota(
                        id=sub['quota']['id'],
                        name=sub['quota']['name'],
                        limits={},  # Simplified for nested quota
                        is_active=True,  # Default values for nested quota
                        reusable=True,
                        subscriptions=[],
                        created_at=self._parse_datetime(quota_data['createdAt']),
                        updated_at=self._parse_datetime(quota_data['updatedAt'])
                    ) if sub.get('quota') else None
                )
                for sub in quota_data['subscriptions']
            ],
            created_at=self._parse_datetime(quota_data['createdAt']),
            updated_at=self._parse_datetime(quota_data['updatedAt'])
        )
    
    def _parse_datetime(self, dt_str: str) -> datetime:
        """
        Parse datetime string from API response.
        
        Args:
            dt_str: Datetime string
            
        Returns:
            datetime object
        """
        return datetime.fromisoformat(dt_str.replace('Z', '+00:00')) 


from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import List, Optional


class BillingFrequency(str, Enum):
    DAILY = 'daily'
    WEEKLY = 'weekly'
    MONTHLY = 'monthly'
    QUARTERLY = 'quarterly'
    YEARLY = 'yearly'


@dataclass
class PaymentMethod:
    id: str
    # Add other payment method fields as needed


@dataclass
class Transaction:
    id: str
    # Add other transaction fields as needed


@dataclass
class Invoice:
    id: str
    # Add other invoice fields as needed


@dataclass
class Subscription:
    id: str
    # Add other subscription fields as needed


@dataclass
class Workspace:
    id: str
    name: str


@dataclass
class BillingAccount:
    id: str
    account_name: str
    billing_address: str
    city: str
    state: str
    country: str
    postal_code: str
    contact_email: str
    contact_phone_number: Optional[str]
    workspace_id: str
    billing_frequency: BillingFrequency
    next_billing_date: datetime
    billing_start_date: datetime
    created_at: datetime
    updated_at: Optional[datetime]
    workspace: Optional[Workspace] = None
    subscriptions: Optional[List[Subscription]] = None
    payment_methods: Optional[List[PaymentMethod]] = None
    transactions: Optional[List[Transaction]] = None
    invoices: Optional[List[Invoice]] = None


@dataclass
class CreateBillingAccountInput:
    account_name: str
    billing_address: str
    city: str
    state: str
    country: str
    postal_code: str
    contact_email: str
    billing_frequency: BillingFrequency
    contact_phone_number: Optional[str] = None


@dataclass
class UpdateBillingAccountInput:
    id: str
    account_name: Optional[str] = None
    billing_address: Optional[str] = None
    city: Optional[str] = None
    state: Optional[str] = None
    country: Optional[str] = None
    postal_code: Optional[str] = None
    contact_email: Optional[str] = None
    contact_phone_number: Optional[str] = None
    billing_frequency: Optional[BillingFrequency] = None


class BillingError(Exception):
    def __init__(self, message: str, code: str = None):
        super().__init__(message)
        self.message = message
        self.code = code


class Billing:
    """
    Billing account management for the Burdenoff Server.
    Handles billing account creation, updates, and retrieval.
    """
    
    def __init__(self, client):
        """
        Initialize Billing module.
        
        Args:
            client: WorkspaceClient instance
        """
        self.client = client
    
    def get_workspace_billing_accounts(self) -> List[BillingAccount]:
        """
        Get all billing accounts for the current workspace.
        
        Returns:
            List of billing accounts for the workspace
            
        Raises:
            BillingError: If there's an error fetching the billing accounts
        """
        query = """
        query GetWorkspaceBillingAccounts {
            getWorkspaceBillingAccounts {
                id
                accountName
                billingAddress
                city
                state
                country
                postalCode
                contactEmail
                contactPhoneNumber
                workspaceId
                billingFrequency
                nextBillingDate
                billingStartDate
                createdAt
                updatedAt
            }
        }
        """
        
        try:
            result = self.client.execute(query)
            
            return [
                self._parse_billing_account(account)
                for account in result['getWorkspaceBillingAccounts']
            ]
        except Exception as e:
            raise BillingError(str(e), getattr(e, 'code', None))
    
    def get_billing_account_details(self, account_id: str) -> BillingAccount:
        """
        Get detailed information about a specific billing account.
        
        Args:
            account_id: ID of the billing account
            
        Returns:
            Detailed billing account information
            
        Raises:
            BillingError: If there's an error fetching the billing account details
        """
        query = """
        query GetBillingAccountDetails($id: ID!) {
            getBillingAccountDetails(id: $id) {
                id
                accountName
                billingAddress
                city
                state
                country
                postalCode
                contactEmail
                contactPhoneNumber
                workspaceId
                billingFrequency
                nextBillingDate
                billingStartDate
                createdAt
                updatedAt
                workspace {
                    id
                    name
                }
                subscriptions {
                    id
                }
                paymentMethods {
                    id
                }
                transactions {
                    id
                }
                invoices {
                    id
                }
            }
        }
        """
        
        variables = {
            "id": account_id
        }
        
        try:
            result = self.client.execute(query, variables)
            account = result['getBillingAccountDetails']
            
            return self._parse_detailed_billing_account(account)
        except Exception as e:
            raise BillingError(str(e), getattr(e, 'code', None))
    
    def get_filtered_billing_accounts(self) -> List[BillingAccount]:
        """
        Get filtered billing accounts (those without active subscriptions).
        
        Returns:
            List of filtered billing accounts
            
        Raises:
            BillingError: If there's an error fetching the filtered billing accounts
        """
        query = """
        query GetFilteredBillingAccount {
            getFilteredBillingAccount {
                id
                accountName
                billingAddress
                city
                state
                country
                postalCode
                contactEmail
                contactPhoneNumber
                workspaceId
                billingFrequency
                nextBillingDate
                billingStartDate
                createdAt
                updatedAt
            }
        }
        """
        
        try:
            result = self.client.execute(query)
            
            return [
                self._parse_billing_account(account)
                for account in result['getFilteredBillingAccount']
            ]
        except Exception as e:
            raise BillingError(str(e), getattr(e, 'code', None))
    
    def create_billing_account(self, input_data: CreateBillingAccountInput) -> bool:
        """
        Create a new billing account.
        
        Args:
            input_data: Input data for creating a billing account
            
        Returns:
            Boolean indicating success
            
        Raises:
            BillingError: If there's an error creating the billing account
        """
        mutation = """
        mutation CreateBillingAccount($input: CreateBillingAccountInput!) {
            createBillingAccount(input: $input)
        }
        """
        
        variables = {
            "input": {
                "accountName": input_data.account_name,
                "billingAddress": input_data.billing_address,
                "city": input_data.city,
                "state": input_data.state,
                "country": input_data.country,
                "postalCode": input_data.postal_code,
                "contactEmail": input_data.contact_email,
                "contactPhoneNumber": input_data.contact_phone_number,
                "billingFrequency": input_data.billing_frequency.value
            }
        }
        
        try:
            result = self.client.execute(mutation, variables)
            return result['createBillingAccount']
        except Exception as e:
            raise BillingError(str(e), getattr(e, 'code', None))
    
    def update_billing_account(self, input_data: UpdateBillingAccountInput) -> bool:
        """
        Update an existing billing account.
        
        Args:
            input_data: Input data for updating a billing account
            
        Returns:
            Boolean indicating success
            
        Raises:
            BillingError: If there's an error updating the billing account
        """
        mutation = """
        mutation UpdateBillingAccount($input: UpdateBillingAccountInput!) {
            updateBillingAccount(input: $input)
        }
        """
        
        update_data = {
            "id": input_data.id
        }
        
        if input_data.account_name is not None:
            update_data["accountName"] = input_data.account_name
        if input_data.billing_address is not None:
            update_data["billingAddress"] = input_data.billing_address
        if input_data.city is not None:
            update_data["city"] = input_data.city
        if input_data.state is not None:
            update_data["state"] = input_data.state
        if input_data.country is not None:
            update_data["country"] = input_data.country
        if input_data.postal_code is not None:
            update_data["postalCode"] = input_data.postal_code
        if input_data.contact_email is not None:
            update_data["contactEmail"] = input_data.contact_email
        if input_data.contact_phone_number is not None:
            update_data["contactPhoneNumber"] = input_data.contact_phone_number
        if input_data.billing_frequency is not None:
            update_data["billingFrequency"] = input_data.billing_frequency.value
        
        variables = {
            "input": update_data
        }
        
        try:
            result = self.client.execute(mutation, variables)
            return result['updateBillingAccount']
        except Exception as e:
            raise BillingError(str(e), getattr(e, 'code', None))
    
    def _parse_billing_account(self, account: dict) -> BillingAccount:
        """
        Parse basic billing account data from GraphQL response.
        
        Args:
            account: Dict containing account data
            
        Returns:
            BillingAccount object
        """
        return BillingAccount(
            id=account['id'],
            account_name=account['accountName'],
            billing_address=account['billingAddress'],
            city=account['city'],
            state=account['state'],
            country=account['country'],
            postal_code=account['postalCode'],
            contact_email=account['contactEmail'],
            contact_phone_number=account['contactPhoneNumber'],
            workspace_id=account['workspaceId'],
            billing_frequency=BillingFrequency(account['billingFrequency']),
            next_billing_date=self._parse_datetime(account['nextBillingDate']),
            billing_start_date=self._parse_datetime(account['billingStartDate']),
            created_at=self._parse_datetime(account['createdAt']),
            updated_at=self._parse_datetime(account['updatedAt']) if account.get('updatedAt') else None
        )
    
    def _parse_detailed_billing_account(self, account: dict) -> BillingAccount:
        """
        Parse detailed billing account data from GraphQL response.
        
        Args:
            account: Dict containing detailed account data
            
        Returns:
            BillingAccount object with related entities
        """
        billing_account = self._parse_billing_account(account)
        
        # Add additional related data
        if account.get('workspace'):
            billing_account.workspace = Workspace(
                id=account['workspace']['id'],
                name=account['workspace']['name']
            )
        
        if account.get('subscriptions'):
            billing_account.subscriptions = [
                Subscription(id=sub['id'])
                for sub in account['subscriptions']
            ]
        
        if account.get('paymentMethods'):
            billing_account.payment_methods = [
                PaymentMethod(id=pm['id'])
                for pm in account['paymentMethods']
            ]
        
        if account.get('transactions'):
            billing_account.transactions = [
                Transaction(id=tx['id'])
                for tx in account['transactions']
            ]
        
        if account.get('invoices'):
            billing_account.invoices = [
                Invoice(id=inv['id'])
                for inv in account['invoices']
            ]
        
        return billing_account
    
    def _parse_datetime(self, dt_str: str) -> datetime:
        """
        Parse datetime string from API response.
        
        Args:
            dt_str: Datetime string
            
        Returns:
            datetime object
        """
        return datetime.fromisoformat(dt_str.replace('Z', '+00:00')) 


from typing import Dict, Optional, Any, List, Tuple, Union
import requests
import jwt
import time
import aiohttp
import uuid # For generating consistent fake IDs if needed


class Auth:
    """
    Authentication operations for the Burdenoff Server.
    Handles JWT verification and context enrichment.
    """
    
    def __init__(self, client):
        """
        Initialize Auth module.
        
        Args:
            client: WorkspaceClient instance (from burdenoff_common.client)
        """
        self.client = client # This is the BurdenoffWorkspaceClient
    
    async def authenticate(self, username: str, password: str) -> Dict[str, Any]:
        """
        Authenticate a user with username and password.
        
        Args:
            username: The user's username or email
            password: The user's password
            
        Returns:
            Dict containing authentication result with the following fields:
            - success: Boolean indicating if authentication was successful
            - token: JWT token if successful, None if failed
            - user: User details if successful, None if failed
        """
        base_url = self.client.endpoint.rsplit('/graphql', 1)[0]
        auth_endpoint = f"{base_url}/authenticate"
        
        payload = {
            "username": username,
            "password": password
        }
        
        try:
            async with aiohttp.ClientSession() as session:
                async with session.post(auth_endpoint, json=payload) as response:
                    if response.status == 200:
                        result = await response.json()
                        # Set the token in the client for subsequent requests
                        if result.get("token"):
                            self.set_token(result["token"])
                        return {
                            "success": True, 
                            "token": result.get("token"),
                            "user": result.get("user")
                        }
                    else:
                        return {"success": False, "token": None, "user": None}
        except Exception as e:
            # Consider logging the error
            return {"success": False, "token": None, "user": None, "error": str(e)}
    
    async def verify_jwt(self, token: str) -> bool:
        """
        Verify if a JWT token is valid by calling the verify-jwt endpoint asynchronously.
        
        Args:
            token: JWT token to verify
            
        Returns:
            Boolean indicating if token is valid
        """
        base_url = self.client.endpoint.rsplit('/graphql', 1)[0]
        verify_endpoint = f"{base_url}/verify-jwt"
        
        params = {"token": token}
        
        try:
            async with aiohttp.ClientSession() as session:
                async with session.get(verify_endpoint, params=params) as response:
                    return response.status == 200
        except Exception as e:
            # Consider logging the error e
            return False
            
    def set_token(self, token: str) -> None:
        """
        Set authentication token in the client.
        
        Args:
            token: JWT authentication token
        """
        self.client.set_token(token)

    async def _get_context(self, token: str, metadata: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        """
        (Internal) Asynchronously decodes a JWT and enriches its payload.
        *** CURRENTLY IMPLEMENTED WITH A FAKE USER DB FOR TESTING/MIGRATION ***
        It will return details for a 'test_user' irrespective of the input token's content,
        though it still expects a syntactically valid JWT for basic decoding to pass.
        Future implementation will fetch real details based on the token.

        Args:
            token: The JWT string. Currently mostly ignored for content, but format is checked.
            metadata: Optional dictionary, may influence future data fetching.

        Returns:
            A dictionary containing the enriched context for the fake 'test_user'.

        Raises:
            jwt.ExpiredSignatureError: If the token has nominally expired (if checked).
            jwt.InvalidTokenError: For JWT format errors or other processing issues.
        """
        try:
            # Basic decode to ensure token is somewhat valid structurally, even if we ignore its content for now.
            # This helps catch completely malformed tokens early.
            decoded_token = jwt.decode(
                token,
                options={"verify_signature": False, "verify_exp": False, "verify_nbf": False, "verify_iat": False}
            )

            # If we wanted to strictly enforce expiry even for the fake user scenario:
            # if 'exp' in decoded_token and decoded_token['exp'] < time.time():
            #     raise jwt.ExpiredSignatureError("Token has expired.")

            # --- FAKE USER DB Implementation --- 
            # Return a consistent set of details for 'test_user'.
            # This helps in migrating API endpoints that expect a certain User model structure.
            
            fake_user_id = str(decoded_token.get("user_id", "test_user_001")) # Try to get a user_id from token, or default
            fake_org_id = str(decoded_token.get("org_id", "test_org_001"))
            fake_tenant_id = str(decoded_token.get("tenant_id", "test_tenant_001"))

            enriched_context = {
                "user": {
                    "id": fake_user_id, # Consistent ID for test_user
                    "username": decoded_token.get("username", "testuser"),
                    "name": decoded_token.get("name", "Test User Default"),
                    "email": decoded_token.get("email", "testuser@example.com"),
                    "roles": decoded_token.get("roles", ["test_role_admin", "test_role_user"]),
                    "is_active": True, # Common field in User models
                    "is_superuser": False, # Example field
                    "profile_picture_url": f"https://example.com/avatars/{fake_user_id}.png",
                    # Add any other fields that existing User models might have, to ease migration
                    # e.g., "email_verified": True,
                    # "created_at": "2023-01-01T10:00:00Z",
                    # "last_login": time.time()
                },
                "organization": {
                    "id": fake_org_id,
                    "name": f"Test Organization for {fake_user_id}",
                    "plan": "premium_test_plan",
                    # e.g., "member_count": 10,
                    # "trial_ends_at": None
                },
                "tenant": {
                    "id": fake_tenant_id,
                    "name": f"Test Tenant for {fake_user_id}",
                    "region": "test_region_1",
                    # e.g., "status": "active"
                },
                "product": {
                    "name": "FluidGrids Workflow Engine (Test Context)",
                    "version_subscribed": "dev-latest",
                    "features_enabled": ["all_test_features", "rbac_beta_test"]
                },
                # Include some top-level claims that might have been in the original JWT payload
                # These are from the decoded_token, so they reflect the *actual* input token if present.
                # This is useful if some code directly accessed decoded_token fields outside of user/org/tenant.
                "jti": decoded_token.get("jti", str(uuid.uuid4())),
                "original_iss": decoded_token.get("iss"),
                "original_aud": decoded_token.get("aud"),
                # Add any other specific claims from decoded_token if they are used directly elsewhere
                # and are not part of the standard excluded_fields or the structured objects above.
                **{k: v for k, v in decoded_token.items() if k not in ['exp', 'iat', 'nbf', 'jti', 'iss', 'sub', 'aud', 'user_id', 'org_id', 'tenant_id', 'name', 'email', 'roles', 'username']} # Non-overlapping generic claims
            }
            # Ensure standard JWT claims that were part of user/org/tenant objects are not duplicated at top level if they came from decoded_token

            return enriched_context

        except jwt.ExpiredSignatureError as e:
            # log.warning(f"Token expired during _get_context (fake DB mode): {e}")
            raise e
        except jwt.InvalidTokenError as e:
            # log.warning(f"Invalid token during _get_context (fake DB mode): {e}")
            raise e
        except Exception as e:
            # log.error(f"Unexpected error during _get_context (fake DB mode): {e}", exc_info=True)
            raise jwt.InvalidTokenError(f"An unexpected error occurred during context processing (fake DB): {e}")

    async def guardRequest(self, jwt_token: str, operation: str, metadata: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        """
        Asynchronously guards an incoming request...
        The 'context' field in the response will be populated by _get_context,
        which currently uses a fake user DB for 'test_user'.
        """
        response = {
            "isAuthenticated": False,
            "context": None,
            "isAuthorized": None,
            "hasSufficientCredits": None,
            "error": None
        }

        try:
            # Step 1: Verify JWT structure and if it's generally valid (e.g., not junk)
            # The verify_jwt might do an external call, which might pass even for an unknown user / expired token for this endpoint.
            # However, for the fake DB scenario, we might primarily rely on _get_context's internal JWT decode.
            # For now, let's assume verify_jwt is a lightweight check or is bypassed if _get_context is in full fake mode.
            
            # If a strict JWT verification (e.g. signature, external check) is still desired before fake data:
            # is_structurally_valid_jwt = await self.verify_jwt(jwt_token)
            # if not is_structurally_valid_jwt:
            #     response["error"] = "JWT verification failed (endpoint check)."
            #     return response
            # response["isAuthenticated"] = True # Tentatively true, _get_context will confirm with its own checks + fake data logic
            
            # For current request: _get_context will decode and provide fake data or raise error
            try:
                token_context = await self._get_context(jwt_token, metadata)
                response["context"] = token_context
                # Since _get_context provides fake data, if it doesn't raise an error, we consider it "authenticated" in this test mode.
                response["isAuthenticated"] = True 
            except jwt.ExpiredSignatureError as e:
                response["isAuthenticated"] = False
                response["error"] = f"Token has expired: {e}"
            except jwt.InvalidTokenError as e:
                response["isAuthenticated"] = False
                response["error"] = f"Invalid token for context retrieval: {e}"
            except Exception as e:
                response["isAuthenticated"] = False
                response["error"] = f"Error retrieving or processing context: {e}"
            
            if not response["isAuthenticated"]:
                 return response
            
            return response

        except Exception as e:
            response["error"] = f"An unexpected error occurred in guardRequest: {e}"
            response["isAuthenticated"] = False
            response["context"] = None
            return response

# Standalone get_context is now removed, it's Auth._get_context 


from typing import Dict, Optional, Any, List, Union


class Workspace:
    """
    Workspace management operations for Burdenoff Server.
    Handles creating, updating, and managing workspaces.
    """
    
    def __init__(self, client):
        """
        Initialize Workspace module.
        
        Args:
            client: WorkspaceClient instance
        """
        self.client = client
    
    def create(self, name: str, tenant_id: str, 
               workspace_type: Optional[str] = None) -> Dict[str, Any]:
        """
        Create a new workspace.
        
        Args:
            name: Workspace name
            tenant_id: ID of the tenant to associate with the workspace
            workspace_type: Optional workspace type/role
            
        Returns:
            Dict containing created workspace details
        """
        query = """
        mutation CreateWorkspace($input: CreateWorkspaceInput!) {
            createWorkspace(input: $input) {
                id
                name
                type
                status
                createdAt
                tenantID
                orgID
            }
        }
        """
        
        input_data = {
            "name": name,
            "tenantID": tenant_id
        }
        
        # Add optional fields if provided
        if workspace_type:
            input_data["type"] = workspace_type
            
        variables = {
            "input": input_data
        }
        
        result = self.client.execute(query, variables)
        return result["createWorkspace"]
    
    def get(self, workspace_id: str) -> Dict[str, Any]:
        """
        Get workspace details by ID.
        
        Args:
            workspace_id: ID of the workspace to retrieve
            
        Returns:
            Dict containing workspace details
        """
        query = """
        query GetWorkspace($id: ID!) {
            workspace(id: $id) {
                id
                name
                type
                status
                createdAt
                updatedAt
                tenantID
                orgID
                workspaceMembers {
                    id
                    userID
                    user {
                        id
                        name
                        email
                    }
                }
            }
        }
        """
        
        variables = {
            "id": workspace_id
        }
        
        result = self.client.execute(query, variables)
        return result["workspace"]
    
    def list(self, tenant_id: Optional[str] = None) -> List[Dict[str, Any]]:
        """
        List available workspaces, optionally filtered by tenant.
        
        Args:
            tenant_id: Optional tenant ID to filter workspaces
            
        Returns:
            List of workspace details
        """
        query = """
        query ListWorkspaces($tenantID: ID) {
            workspaces(tenantID: $tenantID) {
                id
                name
                type
                status
                createdAt
                tenantID
                orgID
            }
        }
        """
        
        variables = {}
        if tenant_id:
            variables["tenantID"] = tenant_id
            
        result = self.client.execute(query, variables)
        return result["workspaces"]
    
    def update(self, workspace_id: str, name: Optional[str] = None, 
               workspace_type: Optional[str] = None, 
               status: Optional[str] = None) -> Dict[str, Any]:
        """
        Update workspace details.
        
        Args:
            workspace_id: ID of the workspace to update
            name: Optional new workspace name
            workspace_type: Optional new workspace type/role
            status: Optional new workspace status
            
        Returns:
            Dict containing updated workspace details
        """
        query = """
        mutation UpdateWorkspace($input: UpdateWorkspaceInput!) {
            updateWorkspace(input: $input) {
                id
                name
                type
                status
                updatedAt
            }
        }
        """
        
        # Start with required ID
        input_data = {
            "id": workspace_id
        }
        
        # Add optional fields if provided
        if name:
            input_data["name"] = name
        if workspace_type:
            input_data["type"] = workspace_type
        if status:
            input_data["status"] = status
            
        variables = {
            "input": input_data
        }
        
        result = self.client.execute(query, variables)
        return result["updateWorkspace"]
    
    def delete(self, workspace_id: str) -> Dict[str, Any]:
        """
        Delete a workspace.
        
        Args:
            workspace_id: ID of the workspace to delete
            
        Returns:
            Dict containing deleted workspace details
        """
        query = """
        mutation DeleteWorkspace($id: ID!) {
            deleteWorkspace(id: $id) {
                id
                name
                status
            }
        }
        """
        
        variables = {
            "id": workspace_id
        }
        
        result = self.client.execute(query, variables)
        return result["deleteWorkspace"]
    
    def archive(self, workspace_id: str) -> Dict[str, Any]:
        """
        Archive a workspace.
        
        Args:
            workspace_id: ID of the workspace to archive
            
        Returns:
            Dict containing archived workspace details
        """
        query = """
        mutation ArchiveWorkspace($id: ID!) {
            archieveWorkspace(id: $id) {
                id
                name
                status
                updatedAt
            }
        }
        """
        
        variables = {
            "id": workspace_id
        }
        
        result = self.client.execute(query, variables)
        return result["archieveWorkspace"]
    
    def reactivate(self, workspace_id: str) -> Dict[str, Any]:
        """
        Reactivate an archived workspace.
        
        Args:
            workspace_id: ID of the workspace to reactivate
            
        Returns:
            Dict containing reactivated workspace details
        """
        query = """
        mutation ReactivateWorkspace($id: ID!) {
            reActivateWorkspace(id: $id) {
                id
                name
                status
                updatedAt
            }
        }
        """
        
        variables = {
            "id": workspace_id
        }
        
        result = self.client.execute(query, variables)
        return result["reActivateWorkspace"]
    
    def invite_user(self, workspace_id: str, email: str, 
                    roles: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        """
        Invite a user to a workspace.
        
        Args:
            workspace_id: ID of the workspace
            email: Email of the user to invite
            roles: Optional roles to assign to the user
            
        Returns:
            Dict containing invitation details
        """
        query = """
        mutation InviteUser($input: InviteUserInput!) {
            inviteUser(input: $input) {
                email
            }
        }
        """
        
        input_data = {
            "workspaceID": workspace_id,
            "email": email
        }
        
        # Add optional roles if provided
        if roles:
            input_data["roles"] = roles
            
        variables = {
            "input": input_data
        }
        
        result = self.client.execute(query, variables)
        return result["inviteUser"]
    
    def handle_invitation(self, invite_token: str, action: str, 
                         user_data: Optional[Dict[str, Any]] = None, 
                         user_id: Optional[str] = None) -> Dict[str, Any]:
        """
        Handle a workspace invitation (accept or reject).
        
        Args:
            invite_token: Invitation token
            action: "accept" or "reject"
            user_data: Optional user registration data (for new users)
            user_id: Optional user ID (for existing users)
            
        Returns:
            Dict containing invitation response details
        """
        query = """
        mutation HandleInvitation($input: InvitationActionInput!) {
            handleInvitation(input: $input) {
                status
                workspace {
                    id
                    name
                }
            }
        }
        """
        
        input_data = {
            "inviteToken": invite_token,
            "action": action
        }
        
        # Add user data if provided (for new users)
        if user_data:
            input_data["user"] = user_data
            
        # Add user ID if provided (for existing users)
        if user_id:
            input_data["userID"] = user_id
            
        variables = {
            "input": input_data
        }
        
        result = self.client.execute(query, variables)
        return result["handleInvitation"]
    
    def switch_workspace(self, workspace_id: str) -> str:
        """
        Switch the active workspace.
        
        Args:
            workspace_id: ID of the workspace to switch to
            
        Returns:
            String confirmation message
        """
        query = """
        mutation SwitchWorkspace($workspaceID: ID!) {
            switchWorkspace(workspaceID: $workspaceID)
        }
        """
        
        variables = {
            "workspaceID": workspace_id
        }
        
        result = self.client.execute(query, variables)
        return result["switchWorkspace"]
    
    def get_members(self, workspace_id: str, page: Optional[int] = None, 
                   items_per_page: Optional[int] = None, 
                   search: Optional[str] = None) -> Dict[str, Any]:
        """
        Get members of a workspace with pagination and search.
        
        Args:
            workspace_id: Workspace ID
            page: Optional page number for pagination
            items_per_page: Optional items per page for pagination
            search: Optional search query for filtering members
            
        Returns:
            Dict containing count and list of workspace members
        """
        query = """
        query GetWorkspaceMembers($workspaceID: ID!, $page: Int, $itemsPerPage: Int, $search: String) {
            getWorkspaceMembers(workspaceID: $workspaceID, page: $page, itemsPerPage: $itemsPerPage, search: $search) {
                count
                members {
                    id
                    userID
                    workspaceID
                    user {
                        id
                        name
                        email
                        profilePicture
                    }
                }
            }
        }
        """
        
        variables = {
            "workspaceID": workspace_id
        }
        
        # Add optional pagination and search parameters
        if page is not None:
            variables["page"] = page
        if items_per_page is not None:
            variables["itemsPerPage"] = items_per_page
        if search:
            variables["search"] = search
            
        result = self.client.execute(query, variables)
        return result["getWorkspaceMembers"] 


from typing import Dict, Optional, Any, List, Union

from gql import Client, gql
from gql.transport.requests import RequestsHTTPTransport
import jwt

from burdenoff_common.auth import Auth
from burdenoff_common.workspace import Workspace
from burdenoff_common.project import Project
from burdenoff_common.billing.payment import Payment
from burdenoff_common.billing.quota import Quota
from burdenoff_common.billing.usage import Usage
from burdenoff_common.billing.plan import Plan
from burdenoff_common.billing.addon import Addon
from burdenoff_common.billing.billing import Billing
from burdenoff_common.permissions import permissions


class WorkspaceClient:
    """
    Main client for the Burdenoff Common SDK.
    Handles authentication and provides access to all services.
    """
    
    def __init__(self, endpoint: str, api_key: Optional[str] = None, token: Optional[str] = None):
        """
        Initialize the Workspace client.
        
        Args:
            endpoint: GraphQL API endpoint URL
            api_key: Optional API key for authentication
            token: Optional JWT token for authentication
        """
        self.endpoint = endpoint
        self.api_key = api_key
        self.token = token
        self._client = None
        self._headers = {}
        
        # Set up authentication headers if provided
        if api_key:
            self._headers["X-API-Key"] = api_key
        if token:
            self._headers["Authorization"] = f"Bearer {token}"
            
        # Initialize service modules
        self.auth = Auth(self)
        self.workspace = Workspace(self)
        self.project = Project(self)
        
        # Initialize billing modules
        self.payment = Payment(self)
        self.quota = Quota(self)
        self.usage = Usage(self)
        self.plan = Plan(self)
        self.addon = Addon(self)
        self.billing = Billing(self)
        
        # Initialize permissions module
        self.permissions = permissions.initialize(endpoint)
        
    @property
    def client(self) -> Client:
        """
        Returns the GQL client with current authentication.
        Lazy-loads the client when needed.
        """
        if not self._client:
            transport = RequestsHTTPTransport(
                url=self.endpoint,
                headers=self._headers,
                use_json=True,
            )
            self._client = Client(transport=transport, fetch_schema_from_transport=True)
        return self._client
    
    def set_token(self, token: str) -> None:
        """
        Set authentication token and update headers.
        
        Args:
            token: JWT authentication token
        """
        self.token = token
        self._headers["Authorization"] = f"{token}"
        # Reset client to apply new headers
        self._client = None
        
    def execute(self, query: str, variables: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        """
        Execute a GraphQL query or mutation.
        
        Args:
            query: GraphQL query or mutation string
            variables: Optional variables for the query
            
        Returns:
            Dict containing the GraphQL response
        """
        if variables is None:
            variables = {}
            
        # Convert query string to gql object
        gql_query = gql(query)
        
        # Execute query
        result = self.client.execute(gql_query, variable_values=variables)
        return result
    
    def is_authenticated(self) -> bool:
        """
        Check if client has valid authentication token.
        
        Returns:
            Boolean indicating if client is authenticated
        """
        if not self.token:
            return False
            
        try:
            # Decode token to check if it's valid
            # This only checks format, not if the token is actually valid on the server
            payload = jwt.decode(self.token, options={"verify_signature": False})
            # Check if token is expired
            if "exp" in payload and payload["exp"] > 0:
                return True
            return False
        except:
            return False 


from burdenoff_common.billing.payment import Payment
from burdenoff_common.billing.quota import Quota
from burdenoff_common.billing.usage import (
    Usage,
    QuotaAssignment,
    PaginatedUsages,
    QuotaAssignmentSummary,
    SubscriptionWithQuotas,
    SubscriptionQuotaUsageWithFilter,
    QuotaCheckResult,
    QuotaUsageAndLimit,
    UsageError
)
from burdenoff_common.billing.plan import Plan
from burdenoff_common.billing.addon import Addon
from burdenoff_common.billing.billing import Billing

__all__ = [
    "Payment", 
    "Quota", 
    "Usage", 
    "Plan", 
    "Addon", 
    "Billing",
    "QuotaAssignment",
    "PaginatedUsages",
    "QuotaAssignmentSummary",
    "SubscriptionWithQuotas",
    "SubscriptionQuotaUsageWithFilter",
    "QuotaCheckResult",
    "QuotaUsageAndLimit",
    "UsageError"
] 


from dataclasses import dataclass
from datetime import datetime
from typing import Dict, List, Optional


@dataclass
class Usage:
    """A usage record for a quota."""
    id: str
    amount: float
    description: str
    last_updated: datetime
    timestamp: datetime
    tags: Optional[Dict] = None


@dataclass
class QuotaAssignment:
    """A quota assignment for a subscription."""
    id: str
    name: str
    limits: Dict
    reusable: bool
    subscription_id: str
    created_at: datetime
    endtime: datetime
    usages: List[Usage]


@dataclass
class PaginatedUsages:
    """Paginated list of usage records."""
    usages: List[Usage]
    total_count: int
    page: int
    page_size: int
    total_pages: int


@dataclass
class QuotaAssignmentSummary:
    """Summary of a quota assignment with usage information."""
    current_usage_sum: int
    usage_count: int
    usages: List[Usage]
    created_at: datetime
    endtime: datetime
    id: str
    name: str
    limits: Dict


@dataclass
class SubscriptionWithQuotas:
    """A subscription with its associated quotas."""
    id: str
    created_at: datetime
    status: str
    quotas: List[QuotaAssignmentSummary]


@dataclass
class SubscriptionQuotaUsageWithFilter:
    """Paginated and filtered subscription quota usage."""
    page_size: int
    total_count: int
    total_pages: int
    page: int
    usages: List[Usage]


@dataclass
class QuotaCheckResult:
    """Result of checking if a quota is exhausted."""
    limit_left: int
    no_limit: bool


@dataclass
class QuotaUsageAndLimit:
    """Current usage and limit information for a quota."""
    limit: int
    limit_left: int
    no_limit: bool


class UsageError(Exception):
    """Custom error class for usage-related errors."""
    def __init__(self, message: str, code: str = None):
        super().__init__(message)
        self.message = message
        self.code = code


class Usage:
    """
    Usage tracking and quota management for the Burdenoff Server.
    Handles tracking resource usage, checking quota limits, and retrieving usage data.
    """
    
    def __init__(self, client):
        """
        Initialize Usage module.
        
        Args:
            client: WorkspaceClient instance
        """
        self.client = client
    
    def add_usage(
        self,
        billing_account_id: str,
        quota_name: str,
        value: int,
        description: str,
        tags: Optional[Dict] = None
    ) -> bool:
        """
        Add usage for a specific quota.
        
        Args:
            billing_account_id (str): The billing account ID
            quota_name (str): Name of the quota
            value (int): Usage value to add
            description (str): Description of the usage
            tags (Optional[Dict]): Optional tags for the usage
            
        Returns:
            bool: True if successful, False if quota is exhausted
            
        Raises:
            UsageError: If there's an error adding the usage
        """
        mutation = """
        mutation AddUsage(
            $billingAccountId: ID!
            $quotaName: String!
            $value: Int!
            $description: String!
            $tags: JSON
        ) {
            addUsage(
                billingAccountId: $billingAccountId
                quotaName: $quotaName
                value: $value
                description: $description
                tags: $tags
            )
        }
        """
        
        variables = {
            "billingAccountId": billing_account_id,
            "quotaName": quota_name,
            "value": value,
            "description": description,
            "tags": tags
        }
        
        try:
            result = self.client.execute(mutation, variables)
            return result["addUsage"]
        except Exception as e:
            # Extract error code if available
            code = getattr(e, "code", None)
            raise UsageError(str(e), code)
    
    def check_quota_exhausted(
        self,
        billing_account_id: str,
        quota_name: str
    ) -> QuotaCheckResult:
        """
        Check if a quota has been exhausted.
        
        Args:
            billing_account_id (str): The billing account ID
            quota_name (str): Name of the quota to check
            
        Returns:
            QuotaCheckResult: Object containing limit left and no limit flag
            
        Raises:
            UsageError: If there's an error checking the quota
        """
        query = """
        query CheckQuotaExhausted($billingAccountId: ID!, $quotaName: String!) {
            checkQuotaExhausted(billingAccountId: $billingAccountId, quotaName: $quotaName) {
                limitLeft
                noLimit
            }
        }
        """
        
        variables = {
            "billingAccountId": billing_account_id,
            "quotaName": quota_name
        }
        
        try:
            result = self.client.execute(query, variables)
            data = result["checkQuotaExhausted"]
            return QuotaCheckResult(
                limit_left=data["limitLeft"],
                no_limit=data["noLimit"]
            )
        except Exception as e:
            code = getattr(e, "code", None)
            raise UsageError(str(e), code)
    
    def get_quota_usage_and_limit(
        self,
        billing_account_id: str,
        quota_name: str
    ) -> QuotaUsageAndLimit:
        """
        Get the current usage and limit for a quota.
        
        Args:
            billing_account_id (str): The billing account ID
            quota_name (str): Name of the quota
            
        Returns:
            QuotaUsageAndLimit: Object containing limit, limit left and no limit flag
            
        Raises:
            UsageError: If there's an error getting the quota usage and limit
        """
        query = """
        query GetQuotaUsageAndLimit($billingAccountId: ID!, $quotaName: String!) {
            getQuotaUsageAndLimit(billingAccountId: $billingAccountId, quotaName: $quotaName) {
                limit
                limitLeft
                noLimit
            }
        }
        """
        
        variables = {
            "billingAccountId": billing_account_id,
            "quotaName": quota_name
        }
        
        try:
            result = self.client.execute(query, variables)
            data = result["getQuotaUsageAndLimit"]
            return QuotaUsageAndLimit(
                limit=data["limit"],
                limit_left=data["limitLeft"],
                no_limit=data["noLimit"]
            )
        except Exception as e:
            code = getattr(e, "code", None)
            raise UsageError(str(e), code)
    
    def get_subscription_usages(
        self,
        billing_account_id: str,
        page: int,
        page_size: int,
        search_query: Optional[str] = None,
        from_date: Optional[datetime] = None,
        to_date: Optional[datetime] = None,
        resource_type: Optional[str] = None,
        entity: Optional[str] = None
    ) -> SubscriptionQuotaUsageWithFilter:
        """
        Get paginated usage records for a subscription.
        
        Args:
            billing_account_id (str): The billing account ID
            page (int): Page number
            page_size (int): Number of items per page
            search_query (Optional[str]): Optional search query
            from_date (Optional[datetime]): Filter usages from this date
            to_date (Optional[datetime]): Filter usages to this date
            resource_type (Optional[str]): Filter by resource type
            entity (Optional[str]): Filter by entity
            
        Returns:
            SubscriptionQuotaUsageWithFilter: Paginated usage records
        """
        query = """
        query GetSubscriptionUsages(
            $billingAccountId: ID!
            $page: Int!
            $pageSize: Int!
            $searchQuery: String
            $fromDate: DateTime
            $toDate: DateTime
            $resourceType: String
            $entity: String
        ) {
            getSubscriptionUsages(
                billingAccountId: $billingAccountId
                page: $page
                pageSize: $pageSize
                searchQuery: $searchQuery
                fromDate: $fromDate
                toDate: $toDate
                resourceType: $resourceType
                entity: $entity
            ) {
                pageSize
                totalCount
                totalPages
                page
                usages {
                    id
                    amount
                    description
                    lastUpdated
                    timestamp
                    tags
                }
            }
        }
        """
        
        variables = {
            "billingAccountId": billing_account_id,
            "page": page,
            "pageSize": page_size,
            "searchQuery": search_query,
            "fromDate": from_date.isoformat() if from_date else None,
            "toDate": to_date.isoformat() if to_date else None,
            "resourceType": resource_type,
            "entity": entity
        }
        
        try:
            result = self.client.execute(query, variables)
            data = result["getSubscriptionUsages"]
            
            # Convert raw data to Usage objects
            usages = [
                Usage(
                    id=u["id"],
                    amount=float(u["amount"]),
                    description=u["description"],
                    last_updated=self._parse_datetime(u["lastUpdated"]),
                    timestamp=self._parse_datetime(u["timestamp"]),
                    tags=u["tags"]
                )
                for u in data["usages"]
            ]
            
            return SubscriptionQuotaUsageWithFilter(
                page_size=data["pageSize"],
                total_count=data["totalCount"],
                total_pages=data["totalPages"],
                page=data["page"],
                usages=usages
            )
        except Exception as e:
            code = getattr(e, "code", None)
            raise UsageError(str(e), code)

    def get_all_quotas_assignment(self, billing_account_id: str) -> List[QuotaAssignment]:
        """
        Get all quota assignments for a billing account.
        
        Args:
            billing_account_id (str): The billing account ID
            
        Returns:
            List[QuotaAssignment]: List of quota assignments
            
        Raises:
            UsageError: If there's an error getting the quota assignments
        """
        query = """
        query GetAllQuotasAssignment($billingAccountId: ID!) {
            getAllQuotasAssignment(billingAccountId: $billingAccountId) {
                id
                name
                limits
                reusable
                subscriptionId
                createdAt
                endtime
                usages {
                    id
                    amount
                    description
                    lastUpdated
                    timestamp
                    tags
                }
            }
        }
        """
        
        variables = {
            "billingAccountId": billing_account_id
        }
        
        try:
            result = self.client.execute(query, variables)
            data = result["getAllQuotasAssignment"]
            
            quotas = []
            for quota_data in data:
                usages = [
                    Usage(
                        id=u["id"],
                        amount=float(u["amount"]),
                        description=u["description"],
                        last_updated=self._parse_datetime(u["lastUpdated"]),
                        timestamp=self._parse_datetime(u["timestamp"]),
                        tags=u["tags"]
                    )
                    for u in quota_data["usages"]
                ]
                
                quota = QuotaAssignment(
                    id=quota_data["id"],
                    name=quota_data["name"],
                    limits=quota_data["limits"],
                    reusable=quota_data["reusable"],
                    subscription_id=quota_data["subscriptionId"],
                    created_at=self._parse_datetime(quota_data["createdAt"]),
                    endtime=self._parse_datetime(quota_data["endtime"]),
                    usages=usages
                )
                quotas.append(quota)
                
            return quotas
        except Exception as e:
            code = getattr(e, "code", None)
            raise UsageError(str(e), code)

    def get_quota_usage_details(
        self,
        quota_assignment_id: str,
        page: int,
        page_size: int,
        search_query: Optional[str] = None,
        from_date: Optional[datetime] = None,
        to_date: Optional[datetime] = None,
        resource_type: Optional[str] = None,
        entity: Optional[str] = None
    ) -> PaginatedUsages:
        """
        Get detailed usage information for a quota assignment.
        
        Args:
            quota_assignment_id (str): ID of the quota assignment
            page (int): Page number
            page_size (int): Number of items per page
            search_query (Optional[str]): Optional search query
            from_date (Optional[datetime]): Filter usages from this date
            to_date (Optional[datetime]): Filter usages to this date
            resource_type (Optional[str]): Filter by resource type
            entity (Optional[str]): Filter by entity
            
        Returns:
            PaginatedUsages: Paginated usage details
            
        Raises:
            UsageError: If there's an error getting the usage details
        """
        query = """
        query GetQuotaUsageDetails(
            $quotaAssignmentId: ID!
            $page: Int!
            $pageSize: Int!
            $searchQuery: String
            $fromDate: DateTime
            $toDate: DateTime
            $resourceType: String
            $entity: String
        ) {
            getQuotaUsageDetails(
                quotaAssignmentId: $quotaAssignmentId
                page: $page
                pageSize: $pageSize
                searchQuery: $searchQuery
                fromDate: $fromDate
                toDate: $toDate
                resourceType: $resourceType
                entity: $entity
            ) {
                usages {
                    id
                    amount
                    description
                    lastUpdated
                    timestamp
                    tags
                }
                totalCount
                page
                pageSize
                totalPages
            }
        }
        """
        
        variables = {
            "quotaAssignmentId": quota_assignment_id,
            "page": page,
            "pageSize": page_size,
            "searchQuery": search_query,
            "fromDate": from_date.isoformat() if from_date else None,
            "toDate": to_date.isoformat() if to_date else None,
            "resourceType": resource_type,
            "entity": entity
        }
        
        try:
            result = self.client.execute(query, variables)
            data = result["getQuotaUsageDetails"]
            
            usages = [
                Usage(
                    id=u["id"],
                    amount=float(u["amount"]),
                    description=u["description"],
                    last_updated=self._parse_datetime(u["lastUpdated"]),
                    timestamp=self._parse_datetime(u["timestamp"]),
                    tags=u["tags"]
                )
                for u in data["usages"]
            ]
            
            return PaginatedUsages(
                usages=usages,
                total_count=data["totalCount"],
                page=data["page"],
                page_size=data["pageSize"],
                total_pages=data["totalPages"]
            )
        except Exception as e:
            code = getattr(e, "code", None)
            raise UsageError(str(e), code)

    def get_subscription_usage_overview(
        self,
        billing_account_id: str,
        page: Optional[int] = None,
        page_size: Optional[int] = None
    ) -> List[SubscriptionWithQuotas]:
        """
        Get usage overview for all subscriptions under a billing account.
        
        Args:
            billing_account_id (str): The billing account ID
            page (Optional[int]): Optional page number
            page_size (Optional[int]): Optional number of items per page
            
        Returns:
            List[SubscriptionWithQuotas]: List of subscriptions with quotas
            
        Raises:
            UsageError: If there's an error getting the subscription usage overview
        """
        query = """
        query GetSubscriptionUsageOverview(
            $billingAccountId: ID!
            $page: Int
            $pageSize: Int
        ) {
            getSubscriptionUsageOverview(
                billingAccountId: $billingAccountId
                page: $page
                pageSize: $pageSize
            ) {
                id
                createdAt
                status
                quotas {
                    id
                    name
                    limits
                    createdAt
                    endtime
                    currentUsageSum
                    usageCount
                    usages {
                        id
                        amount
                        description
                        lastUpdated
                        timestamp
                        tags
                    }
                }
            }
        }
        """
        
        variables = {
            "billingAccountId": billing_account_id,
            "page": page,
            "pageSize": page_size
        }
        
        try:
            result = self.client.execute(query, variables)
            data = result["getSubscriptionUsageOverview"]
            
            subscriptions = []
            for sub_data in data:
                quotas = []
                for quota_data in sub_data["quotas"]:
                    usages = [
                        Usage(
                            id=u["id"],
                            amount=float(u["amount"]),
                            description=u["description"],
                            last_updated=self._parse_datetime(u["lastUpdated"]),
                            timestamp=self._parse_datetime(u["timestamp"]),
                            tags=u["tags"]
                        )
                        for u in quota_data["usages"]
                    ]
                    
                    quota = QuotaAssignmentSummary(
                        id=quota_data["id"],
                        name=quota_data["name"],
                        limits=quota_data["limits"],
                        created_at=self._parse_datetime(quota_data["createdAt"]),
                        endtime=self._parse_datetime(quota_data["endtime"]),
                        current_usage_sum=quota_data["currentUsageSum"],
                        usage_count=quota_data["usageCount"],
                        usages=usages
                    )
                    quotas.append(quota)
                
                subscription = SubscriptionWithQuotas(
                    id=sub_data["id"],
                    created_at=self._parse_datetime(sub_data["createdAt"]),
                    status=sub_data["status"],
                    quotas=quotas
                )
                subscriptions.append(subscription)
                
            return subscriptions
        except Exception as e:
            code = getattr(e, "code", None)
            raise UsageError(str(e), code)
    
    def _parse_datetime(self, dt_str: str) -> datetime:
        """
        Parse a datetime string into a datetime object.
        
        Args:
            dt_str (str): ISO format datetime string
            
        Returns:
            datetime: Parsed datetime object
        """
        return datetime.fromisoformat(dt_str.replace('Z', '+00:00')) 